//+------------------------------------------------------------------+
//|                                              Order manager class |
//|                            Copyright 2015, Kristoffer Klevenberg |
//|                                             https://www.mql5.com |
//+------------------------------------------------------------------+
#property copyright "Copyright 2015, Kristoffer Klevenberg"
#property link      "https://www.mql5.com"
#property version   "1.00"
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
class OrdersManager{
   private:
      MqlTradeRequest _tradeRequest; 
      MqlTradeResult _tradeResult; 
      MqlTradeCheckResult _tradeCheck; 
      
      string getRetcodeDescription(int retcode)
      { 
         string str; 
         
         switch(retcode) 
           { 
            case TRADE_RETCODE_REQUOTE: str="Requote"; break; 
            case TRADE_RETCODE_REJECT: str="Request rejected"; break; 
            case TRADE_RETCODE_CANCEL: str="Request cancelled by trader"; break; 
            case TRADE_RETCODE_PLACED: str="Order is placed"; break; 
            case TRADE_RETCODE_DONE: str="Request is executed"; break; 
            case TRADE_RETCODE_DONE_PARTIAL: str="Request is executed partially"; break; 
            case TRADE_RETCODE_ERROR: str="Request processing error"; break; 
            case TRADE_RETCODE_TIMEOUT: str="Request is cancelled because of a time out";break; 
            case TRADE_RETCODE_INVALID: str="Invalid request"; break; 
            case TRADE_RETCODE_INVALID_VOLUME: str="Invalid request volume"; break; 
            case TRADE_RETCODE_INVALID_PRICE: str="Invalid request price"; break; 
            case TRADE_RETCODE_INVALID_STOPS: str="Invalid request stops"; break; 
            case TRADE_RETCODE_TRADE_DISABLED: str="Trading is forbidden"; break; 
            case TRADE_RETCODE_MARKET_CLOSED: str="Market is closed"; break; 
            case TRADE_RETCODE_NO_MONEY: str="Insufficient funds for request execution"; break; 
            case TRADE_RETCODE_PRICE_CHANGED: str="Prices have changed"; break; 
            case TRADE_RETCODE_PRICE_OFF: str="No quotes for request processing"; break; 
            case TRADE_RETCODE_INVALID_EXPIRATION: str="Invalid order expiration date in the request"; break; 
            case TRADE_RETCODE_ORDER_CHANGED: str="Order state has changed"; break; 
            case TRADE_RETCODE_TOO_MANY_REQUESTS: str="Too many requests"; break; 
            case TRADE_RETCODE_NO_CHANGES: str="No changes in the request"; break; 
            case TRADE_RETCODE_SERVER_DISABLES_AT: str="Autotrading is disabled by the server"; break; 
            case TRADE_RETCODE_CLIENT_DISABLES_AT: str="Autotrading is disabled by the client terminal"; break; 
            case TRADE_RETCODE_LOCKED: str="Request is blocked for processing"; break; 
            case TRADE_RETCODE_FROZEN: str="Order or position has been frozen"; break; 
            case TRADE_RETCODE_INVALID_FILL: str="Unsupported type of order execution for the balance is specified "; break; 
            case TRADE_RETCODE_CONNECTION: str="No connection with trade server"; break; 
            case TRADE_RETCODE_ONLY_REAL: str="Operation is allowed only for real accounts"; break; 
            case TRADE_RETCODE_LIMIT_ORDERS: str="Limit for the number of pending orders has been reached"; break; 
            case TRADE_RETCODE_LIMIT_VOLUME: str="Limit for orders and positions volume for this symbol has been reached"; break; 
            default: str="Unknown result"; 
         };
         return(str); 
      }; 
      
      
   public:
      void _clearStructures()
      { 
         ZeroMemory(_tradeRequest); 
         ZeroMemory(_tradeResult); 
         ZeroMemory(_tradeCheck); 
      }; 
      //Constructor
      OrdersManager(){};
      //Deconstructor
      ~OrdersManager(){};
      
      
      //GET PROFIT OF SYMBOL
      double GetBalanceFromSymbol(string symbol){
         PositionSelect(symbol);
         double symbolProfit = PositionGetDouble(POSITION_PROFIT);
         return symbolProfit;
      };
      
      
      //GET OPEN TRADES FROM MAGIC NUMBER
      int OpenPositionsFromMagicNumber(long magicNumber){
         int openTrades = 0;
         int number;
         int positions = PositionsTotal();
         if (positions > 0){
            for(int x=0;x<=positions; x++){
               PositionSelect(x);
               number = PositionGetInteger(POSITION_MAGIC);
               if (number == magicNumber)
                  openTrades++;
            }
            return openTrades-1;
         }else{
            return 0;
         }
      
      };
      
      //GET TOTAL PROFIT
      double OpenBalanceTotal(){
         double profitTotal = 0;
         double profit = 0;
         
         int positions = PositionsTotal();
         for(int x=0;x<=positions; x++)
         {
            string thisSymbol = PositionGetSymbol(x);
            if (StringLen(thisSymbol) > 1){
               profit = PositionGetDouble(POSITION_PROFIT);
               profitTotal = profitTotal+profit;
            }
         }
         return profitTotal;
      };
      
      
      
      //
      // OPEN POSITIONS
      //
      bool OpenPosition
      (  
         string buyOrSell,        // BUY OR SELL 
         const string symbol,        // deal trading pair 
         double price,                 // price
         double volume,              // volume
         uint deviation,             // slippage 
         double dStopLoss,           // Stop loss (in price chart units) 
         double dTakeprofit,          // Take profit (in price chart units) 
         long magic
      ){  
      
         
         //---- nulling the structures 
         _clearStructures();
         
         ENUM_ORDER_TYPE orderType;
         ENUM_SYMBOL_INFO_DOUBLE priceSymbol;
         
         if (buyOrSell == "BUY"){
            orderType=ORDER_TYPE_BUY; 
            priceSymbol=SYMBOL_ASK;
         }else{
            orderType=ORDER_TYPE_SELL;
            priceSymbol=SYMBOL_BID; 
         }
         
         if(volume<=0) 
         { 
            Print(__FUNCTION__,"(): Incorrect volume for a trade request structure"); 
            return(false); 
         } 
         
         //---- initializing structure of the Mql_tradeRequest to open BUY position 
         _tradeRequest.type   = orderType; 
         _tradeRequest.price  = price; 
         _tradeRequest.action = TRADE_ACTION_DEAL; 
         _tradeRequest.symbol = symbol; 
         _tradeRequest.volume = volume; 
         _tradeRequest.magic=magic;
         _tradeRequest.sl=dStopLoss; 
         _tradeRequest.tp=dTakeprofit; 
         _tradeRequest.deviation=deviation; 
         _tradeRequest.type_filling=ORDER_FILLING_FOK; 
         
         //---- Checking correctness of a trade request 
         if(!OrderCheck(_tradeRequest,_tradeCheck)) 
         { 
            Print(__FUNCTION__,"(): Incorrect data for a trade request structure!"); 
            Print(__FUNCTION__,"(): OrderCheck(): ",getRetcodeDescription(_tradeCheck.retcode)); 
            return(false); 
         } 
         
         string comment=""; 
         StringConcatenate(comment,"<<< ============ ",__FUNCTION__,"(): Opening ",buyOrSell," position at ",symbol," ============ >>>"); 
         Print(comment); 
         
         //---- open BUY position and check the result of trade request 
         if(!OrderSend(_tradeRequest,_tradeResult) || _tradeResult.retcode!=TRADE_RETCODE_DONE){ 
            Print(__FUNCTION__,"(): Unable to perform a deal!"); 
            Print(__FUNCTION__,"(): OrderSend(): ",getRetcodeDescription(_tradeResult.retcode)); 
            return(false); 
         }else{ 
            if(_tradeResult.retcode==TRADE_RETCODE_DONE){  
               comment=""; 
            }else{ 
               Print(__FUNCTION__,"(): Unable to perform a deal!"); 
               Print(__FUNCTION__,"(): OrderSend(): ",getRetcodeDescription(_tradeResult.retcode)); 
            } 
            //---- 
            return(true);
         } 
      }
      
      //
      // CLOSE POSITIONS
      //
      
      bool PositionClose 
      ( 
         string buyOrSell,       // BUY OR SELL 
         const string symbol,       // deal trading pair 
         uint deviation             // slippage 
      ){
      
         //---- nulling the structures 
         _clearStructures();
         
         ENUM_ORDER_TYPE orderType;
         ENUM_POSITION_TYPE posType;
         ENUM_SYMBOL_INFO_DOUBLE priceSymbol;
         
         
         if (buyOrSell == "BUY"){
            orderType=ORDER_TYPE_SELL;
            posType=POSITION_TYPE_BUY;
            priceSymbol=SYMBOL_BID;
         }else{
            orderType=ORDER_TYPE_BUY;
            posType=POSITION_TYPE_SELL;  
            priceSymbol=SYMBOL_ASK;
         }
            
         
         //---- check, if there is a BUY position 
         if(PositionSelect(symbol)){ 
            if(PositionGetInteger(POSITION_TYPE)!=posType) return(false); 
         } 
         else return(false); 
         
         //---- getting calculation data 
         double volume=PositionGetDouble(POSITION_VOLUME); 
         double MaxLot=SymbolInfoDouble(symbol,SYMBOL_VOLUME_MAX); 
         double price=SymbolInfoDouble(symbol,priceSymbol); 
         if(!volume || !MaxLot || !price) return(false); 
         
         //---- checking the lot for the maximum allowable value       
         if(volume>MaxLot) volume=MaxLot; 
         
         //---- initializing structure of the MqlTradeRequest to close BUY position 
         _tradeRequest.type   = orderType; 
         _tradeRequest.price  = price; 
         _tradeRequest.action = TRADE_ACTION_DEAL; 
         _tradeRequest.symbol = symbol; 
         _tradeRequest.volume = volume; 
         _tradeRequest.sl = 0.0; 
         _tradeRequest.tp = 0.0; 
         _tradeRequest.deviation=deviation; 
         _tradeRequest.type_filling=ORDER_FILLING_FOK; 
         
         //---- Checking correctness of a trade request 
         if(!OrderCheck(_tradeRequest,_tradeCheck)){ 
            Print(__FUNCTION__,"(): Incorrect data for a trade request structure!"); 
            Print(__FUNCTION__,"(): OrderCheck(): ",getRetcodeDescription(_tradeCheck.retcode)); 
            return(false); 
         } 
         
         //----     
         string comment=""; 
         StringConcatenate(comment,"<<< ============ ",__FUNCTION__,"(): Closing ",buyOrSell," position at ",symbol," ============ >>>"); 
         Print(comment); 
         
         //---- send order to close position to trade server 
         if(!OrderSend(_tradeRequest,_tradeResult) || _tradeResult.retcode!=TRADE_RETCODE_DONE){ 
            Print(__FUNCTION__,"(): Unable to close the position!"); 
            Print(__FUNCTION__,"(): OrderSend(): ",getRetcodeDescription(_tradeResult.retcode)); 
            return(false); 
         }else{
            if(_tradeResult.retcode==TRADE_RETCODE_DONE){ 
               comment="";  
            }else{ 
               Print(__FUNCTION__,"(): Unable to close the position!"); 
               Print(__FUNCTION__,"(): OrderSend(): ",getRetcodeDescription(_tradeResult.retcode)); 
            } 
         
         } 
         return(true); 
      } 
      
};
