//+------------------------------------------------------------------+
//|                                       Triangular Arbitrage Class |
//|                            Copyright 2015, Kristoffer Klevenberg |
//|                                             https://www.mql5.com |
//+------------------------------------------------------------------+
#property copyright "Copyright 2015, Kristoffer Klevenberg"
#property link      "https://www.mql5.com"
#property version   "1.00"
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
class TriangularAbitrage{
   private:
      string _currencyMain;
      string _currencyHigh;
      string _currencyLow;
      double _priceHighAsk;
      double _priceMainAsk;
      double _priceLowAsk;
      double _priceHighBid;
      double _priceMainBid;
      double _priceLowBid;
      string _magicNumber;
      string _tradeType;
      double ProfitTotalSave;
      double BuyArbitrageMax;
      double SellArbitrageMax;
      
   public:
      //Constructor
      TriangularAbitrage(string c1, string c2, string c3,string magic){
         _currencyMain = c1;
         _currencyHigh = c2;
         _currencyLow = c3;
         _magicNumber = magic;
         BuyArbitrageMax = -10;
         SellArbitrageMax = -10;
      };
      
      //Deconstructor
      ~TriangularAbitrage(){};
      
      
      //Set market prizes    
      bool SetMarketPrizes(){
         _priceHighAsk = SymbolInfoDouble(_currencyHigh,SYMBOL_ASK);
         _priceMainAsk = SymbolInfoDouble(_currencyMain,SYMBOL_ASK);
         _priceLowAsk = SymbolInfoDouble(_currencyLow,SYMBOL_ASK);
         
         _priceHighBid = SymbolInfoDouble(_currencyHigh,SYMBOL_BID);
         _priceMainBid = SymbolInfoDouble(_currencyMain,SYMBOL_BID);
         _priceLowBid = SymbolInfoDouble(_currencyLow,SYMBOL_BID);
         return true;
      };
      
      //get Buy Arbitrage
      double GetBuyPips(){
         double Indirect,pips;
         Indirect = (_priceMainBid)*(_priceLowBid);
         pips = (Indirect-_priceHighAsk)*1000;
         return pips;
      };
      
      double GetBuyPercentage(){
         double Indirect,percentage;
         Indirect = (_priceMainBid)*(_priceLowBid);
         percentage = (Indirect-_priceHighAsk)/_priceHighAsk*100000;
         return percentage;
      };
      
      double GetSellPercentage(){
         
         double Indirect,percentage;
         Indirect = (_priceMainAsk)*(_priceLowAsk);
         percentage = (_priceHighBid-Indirect)/_priceHighBid*100000;
         return percentage;
      };
      
      
      //get Sell Arbitrage
      double GetSellPips(){
         double Indirect,pips;
         Indirect = (_priceMainAsk)*(_priceLowAsk);
         pips = (_priceHighBid-Indirect)*1000;
         return pips;
      };  
      
      bool Trade(){
         //------------------
         //set currency
         SetMarketPrizes();
         //get arbitrage
         double BuyArbitrage = GetSellPercentage();
         double SellArbitrage = GetBuyPercentage();         
         //get profit
         double ProfitTotal = MyOrders.OpenBalanceTotal(); 
         //get open orders
         int openOrders = MyOrders.OpenPositionsFromMagicNumber(_magicNumber);
         //------------------
         if (BuyArbitrage > BuyArbitrageMax){ BuyArbitrageMax = BuyArbitrage;}
         if (SellArbitrage > SellArbitrageMax){ SellArbitrageMax = SellArbitrage;}
         Print("CODE:",_magicNumber,"   - BUY: (",BuyArbitrage,") MAX(",BuyArbitrageMax,") --- SELL (",SellArbitrage,") MAX(",SellArbitrageMax,")");
         
         //Set minimum profit
         double commission = ceil( (RISK_COMMISSION/3) );
         double profitNeeded = commission*2;
         
         
         //open trades
         if (openOrders == 0){
         
            if( (SellArbitrage > TRIGGER_VARIABLE_SELL ) )
            {
               Print("Trigger OPEN - Arbitrage: (",SellArbitrage,")","  -",_magicNumber);
               MyOrders.OpenPosition("BUY",_currencyHigh,_priceHighAsk,LOT_VOLUME,RISK_DEVIATION,RISK_STOP_LOSS,RISK_TAKE_PROFIT,_magicNumber);
               MyOrders.OpenPosition("SELL",_currencyMain,_priceMainBid,LOT_VOLUME,RISK_DEVIATION,RISK_STOP_LOSS,RISK_TAKE_PROFIT,_magicNumber);
               MyOrders.OpenPosition("SELL",_currencyLow,_priceLowBid,LOT_VOLUME,RISK_DEVIATION,RISK_STOP_LOSS,RISK_TAKE_PROFIT,_magicNumber);
               _tradeType = "BUY";
            }
            
            if( ( BuyArbitrage > TRIGGER_VARIABLE_BUY ) ){
            
               Print("Trigger OPEN2 - Arbitrage: (",BuyArbitrage,")","  -",_magicNumber);
               MyOrders.OpenPosition("SELL",_currencyHigh,_priceHighBid,LOT_VOLUME,RISK_DEVIATION,RISK_STOP_LOSS,RISK_TAKE_PROFIT,_magicNumber);
               MyOrders.OpenPosition("BUY",_currencyMain,_priceMainAsk,LOT_VOLUME,RISK_DEVIATION,RISK_STOP_LOSS,RISK_TAKE_PROFIT,_magicNumber);
               MyOrders.OpenPosition("BUY",_currencyLow,_priceLowAsk,LOT_VOLUME,RISK_DEVIATION,RISK_STOP_LOSS,RISK_TAKE_PROFIT,_magicNumber);
                
               _tradeType = "SELL";
            }
            
         }else{
            //SWAP TO BUY
            if( (( _tradeType == "SELL" ) && ( SellArbitrage > TRIGGER_VARIABLE_SELL ) && ( ProfitTotal > profitNeeded ) )){
               Print("Trigger Swap - Profit: (",ProfitTotal,") - Commission: (",commission,")","  -",_magicNumber);
               MyOrders.OpenPosition("BUY",_currencyHigh,_priceHighAsk,LOT_VOLUME*2,RISK_DEVIATION,RISK_STOP_LOSS,RISK_TAKE_PROFIT,_magicNumber);
               MyOrders.OpenPosition("SELL",_currencyMain,_priceMainBid,LOT_VOLUME*2,RISK_DEVIATION,RISK_STOP_LOSS,RISK_TAKE_PROFIT,_magicNumber);
               MyOrders.OpenPosition("SELL",_currencyLow,_priceLowBid,LOT_VOLUME*2,RISK_DEVIATION,RISK_STOP_LOSS,RISK_TAKE_PROFIT,_magicNumber);
               _tradeType = "BUY";
            }
            
            //SWAP TO SELL
            if( (( _tradeType == "BUY" ) && ( BuyArbitrage > TRIGGER_VARIABLE_BUY )&& ( ProfitTotal > profitNeeded )) ){
               Print("Trigger Swap - Profit: (",ProfitTotal,") - Commission: (",commission,")","  -",_magicNumber);
               MyOrders.OpenPosition("SELL",_currencyHigh,_priceHighBid,LOT_VOLUME*2,RISK_DEVIATION,RISK_STOP_LOSS,RISK_TAKE_PROFIT,_magicNumber);
               MyOrders.OpenPosition("BUY",_currencyMain,_priceMainAsk,LOT_VOLUME*2,RISK_DEVIATION,RISK_STOP_LOSS,RISK_TAKE_PROFIT,_magicNumber);
               MyOrders.OpenPosition("BUY",_currencyLow,_priceLowAsk,LOT_VOLUME*2,RISK_DEVIATION,RISK_STOP_LOSS,RISK_TAKE_PROFIT,_magicNumber);             
               _tradeType = "SELL";
            }
            
            
         }
         
         return false;
      };
      
      
      
      
};